'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readTokenDot = readTokenDot;
exports.readTokenSlash = readTokenSlash;
exports.readTokenMultModuloExp = readTokenMultModuloExp;
exports.readTokenPipeAmp = readTokenPipeAmp;
exports.readTokenCaret = readTokenCaret;
exports.readTokenPlusMin = readTokenPlusMin;
exports.readTokenLtGt = readTokenLtGt;
exports.readTokenEqExcl = readTokenEqExcl;

var _acorn = require('acorn');

var _mapping = require('../mapping');

var mapping = _interopRequireWildcard(_mapping);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var multiCompare = mapping.multiCompare,
    dots = mapping.dots,
    lts = mapping.lts,
    gts = mapping.gts,
    slashs = mapping.slashs,
    eqs = mapping.eqs;
// ### Token reading

// This is the function that is called to fetch the next token. It
// is somewhat obscure, because it works in character codes rather
// than characters, and because operator parsing has been inlined
// into it.
//
// All in the name of speed.
//

function readTokenDot() {
  return function ha() {
    var next = this.input.charCodeAt(this.pos + 1);
    if (next >= 65296 && next <= 65305) return this.readNumber(true);
    var next2 = this.input.charCodeAt(this.pos + 2);
    if (this.options.ecmaVersion >= 6 && multiCompare(dots, next, next2)) {
      this.pos += 3;
      return this.finishToken(_acorn.tokTypes.ellipsis);
    }
    this.pos += 1;
    return this.finishToken(_acorn.tokTypes.dot);
  };
}

function readTokenSlash() {
  return function ha() {
    // '/'
    var next = this.input.charCodeAt(this.pos + 1);
    if (this.exprAllowed) {
      this.pos += 1;return this.readRegexp();
    }
    if (multiCompare(slashs, next)) return this.finishOp(_acorn.tokTypes.assign, 2);
    return this.finishOp(_acorn.tokTypes.slash, 1);
  };
}

function readTokenMultModuloExp() {
  return function ha(code) {
    // '%*'
    var next = this.input.charCodeAt(this.pos + 1);
    var size = 1;
    var tokentype = code === 215 || code === 65290 ? _acorn.tokTypes.star : _acorn.tokTypes.modulo;

    // exponentiation operator ** and **=
    if (this.options.ecmaVersion >= 7 && (next === 215 || next === 65290)) {
      size += 1;
      tokentype = _acorn.tokTypes.starstar;
      next = this.input.charCodeAt(this.pos + 2);
    }

    if (next === 65309) return this.finishOp(_acorn.tokTypes.assign, size + 1);
    return this.finishOp(tokentype, size);
  };
}

function readTokenPipeAmp() {
  return function ha(code) {
    // '|&'
    var next = this.input.charCodeAt(this.pos + 1);
    if (next === code) return this.finishOp(code === 65372 ? _acorn.tokTypes.logicalOR : _acorn.tokTypes.logicalAND, 2);
    if (next === 65309) return this.finishOp(_acorn.tokTypes.assign, 2);
    return this.finishOp(code === 65372 ? _acorn.tokTypes.bitwiseOR : _acorn.tokTypes.bitwiseAND, 1);
  };
}

function readTokenCaret() {
  return function ha() {
    // '^'
    var next = this.input.charCodeAt(this.pos + 1);
    if (next === 65309) return this.finishOp(_acorn.tokTypes.assign, 2);
    return this.finishOp(_acorn.tokTypes.bitwiseXOR, 1);
  };
}

function readTokenPlusMin() {
  return function ha(code) {
    // '+-'
    var next = this.input.charCodeAt(this.pos + 1);
    if (next === code) {
      return this.finishOp(_acorn.tokTypes.incDec, 2);
    }
    if (next === 65309) return this.finishOp(_acorn.tokTypes.assign, 2);
    return this.finishOp(_acorn.tokTypes.plusMin, 1);
  };
}

function readTokenLtGt() {
  return function ha(code) {
    // '<>'
    var next = this.input.charCodeAt(this.pos + 1);
    var size = 1;
    if (next === code) {
      size = gts.indexOf(code) >= 0 && this.input.charCodeAt(this.pos + 2) === code ? 3 : 2;
      if (lts.indexOf(this.input.charCodeAt(this.pos + size)) >= 0) {
        return this.finishOp(_acorn.tokTypes.assign, size + 1);
      }
      return this.finishOp(_acorn.tokTypes.bitShift, size);
    }
    if (lts.indexOf(next) >= 0) size = 2;
    return this.finishOp(_acorn.tokTypes.relational, size);
  };
}

function readTokenEqExcl() {
  return function ha(code) {
    // '=!'
    var next = this.input.charCodeAt(this.pos + 1);
    if (multiCompare(eqs, next)) {
      return this.finishOp(_acorn.tokTypes.equality, this.input.charCodeAt(this.pos + 2) === 65309 ? 3 : 2);
    }
    if (multiCompare(eqs, code) && multiCompare(gts, next) && this.options.ecmaVersion >= 6) {
      // '=>'
      this.pos += 2;
      return this.finishToken(_acorn.tokTypes.arrow);
    }
    return this.finishOp(multiCompare(eqs, code) ? _acorn.tokTypes.eq : _acorn.tokTypes.prefix, 1);
  };
}